@@ -1,7 +1,7 @@
 /**
  * mobile.js - Base mobile application logic.
  *
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -155,7 +155,7 @@ var KronolithMobile = {
     insertEvents: function(dates, view, cal)
     {
         var key = dates[0].dateString() + dates[1].dateString() + view + cal,
-        d = [dates[0].clone(), dates[1].clone()], date, events, list, key, day;
+        d = [dates[0].clone(), dates[1].clone()], date, events, list, day;
 
         // Make sure all calendars are loaded before rendering the view.
         // @TODO: Implement LIFO queue as in kronolith.js
@@ -473,7 +473,7 @@ var KronolithMobile = {
         } else if (today.clone().addDays(1).dateString() == d.dateString()) {
           text = Kronolith.text.tomorrow;
         } else {
-          text = d.toString('ddd') + ' ' + d.toString('d')
+          text = d.toString('ddd') + ' ' + d.toString('d');
         }
         $('#kronolithDayDetailHeader h3').text(text);
         $('.kronolith-selected').removeClass('kronolith-selected');
@@ -788,6 +788,9 @@ var KronolithMobile = {
         $('#kronolith-minical').on('click', 'td', function(e) {
             KronolithMobile.selectMonthDay($(e.target).jqmData('date'));
         });
+
+        // Load initial view.
+        KronolithMobile.loadPage();
     }
 };
 
