@@ -282,14 +282,14 @@ class Kronolith_Api extends Horde_Registry_Api
                 $content_type = 'text/calendar';
             }
         } else {
-            throw new Kronolith_Exception("Invalid calendar data supplied.");
+            throw new Kronolith_Exception(_("Invalid calendar data supplied."));
         }
 
         if (!Kronolith::hasPermission($calendar, Horde_Perms::EDIT)) {
             // FIXME: Should we attempt to create a calendar based on the
             // filename in the case that the requested calendar does not
             // exist?
-            throw new Kronolith_Exception("Calendar does not exist or no permission to edit");
+            throw new Kronolith_Exception(_("Calendar does not exist or no permission to edit"));
         }
 
         // Store all currently existings UIDs. Use this info to delete UIDs not
@@ -402,7 +402,7 @@ class Kronolith_Api extends Horde_Registry_Api
 
         if (!(count($parts) == 2 || count($parts) == 3) ||
             !Kronolith::hasPermission($calendarId, Horde_Perms::DELETE)) {
-                throw new Kronolith_Exception("Calendar does not exist or no permission to delete");
+                throw new Kronolith_Exception(_("Calendar does not exist or no permission to delete"));
             }
 
         if (count($parts) == 3) {
@@ -440,15 +440,52 @@ class Kronolith_Api extends Horde_Registry_Api
         return array_keys(Kronolith::listInternalCalendars($owneronly, $permission));
     }
 
+    /**
+     * Returns a list of available sources.
+     *
+     * @param boolean $writeable  If true, limits to writeable sources.
+     * @param boolean $sync_only  Only include syncable sources.
+     *
+     * @return array  An array of the available sources. Keys are source IDs,
+     *                values are source titles.
+     * @since 4.2.0
+     */
+    public function sources($writeable = false, $sync_only = false)
+    {
+        $out = array();
+
+        foreach (Kronolith::listInternalCalendars(false, $writeable ? Horde_Perms::EDIT : Horde_Perms::READ) as $id => $data) {
+            $out[$id] = $data->get('name');
+        }
+
+        if ($sync_only) {
+            $syncable = Kronolith::getSyncCalendars();
+            $out = array_intersect_key($out, array_flip($syncable));
+        }
+
+        return $out;
+    }
+
+    /**
+     * Retrieve the UID for the current user's default calendar.
+     *
+     * @return string  UID.
+     * @since 4.2.0
+     */
+    public function getDefaultShare()
+    {
+        return Kronolith::getDefaultCalendar(Horde_Perms::EDIT, true);
+    }
+
     /**
      * Returns the ids of all the events that happen within a time period.
      * Only includes recurring events once per time period, and does not include
      * events that represent exceptions, making this method useful for syncing
      * purposes. For more control, use the listEvents method.
      *
-     * @param string $calendars    The calendar to check for events.
-     * @param object $startstamp   The start of the time range.
-     * @param object $endstamp     The end of the time range.
+     * @param string|array $calendars    The calendar to check for events.
+     * @param object $startstamp         The start of the time range.
+     * @param object $endstamp           The end of the time range.
      *
      * @return array  The event ids happening in this time period.
      * @throws Kronolith_Exception
@@ -465,7 +502,7 @@ class Kronolith_Api extends Horde_Registry_Api
         $results = array();
         foreach ($calendars as $calendar) {
             if (!Kronolith::hasPermission($calendar, Horde_Perms::READ)) {
-                Horde::logMessage(sprintf(
+                Horde::log(sprintf(
                     _("Permission Denied or Calendar Not Found: %s - skipping."),
                     $calendar));
                 continue;
@@ -480,7 +517,7 @@ class Kronolith_Api extends Horde_Registry_Api
                 );
                 Kronolith::mergeEvents($results, $events);
             } catch (Kronolith_Exception $e) {
-                Horde::logMessage($e);
+                Horde::log($e);
             }
         }
         $uids = array();
@@ -552,15 +589,25 @@ class Kronolith_Api extends Horde_Registry_Api
      * @param boolean $isModSeq          If true, $timestamp and $end are
      *                                   modification sequences and not
      *                                   timestamps. @since 4.1.1
+     * @param string|array $calendars    The sources to check. @since 4.2.0
      *
      * @return array  An hash with 'add', 'modify' and 'delete' arrays.
      * @throws Horde_Exception_PermissionDenied
      * @throws Kronolith_Exception
      */
-    public function getChanges($start, $end, $ignoreExceptions = true, $isModSeq = false)
+    public function getChanges(
+        $start, $end, $ignoreExceptions = true, $isModSeq = false, $calendars = null)
     {
         // Only get the calendar once
-        $cs = Kronolith::getSyncCalendars();
+        if (is_null($calendars)) {
+            $cs = Kronolith::getSyncCalendars();
+        } else {
+            if (!is_array($calendars)) {
+                $calendars = array($calendars);
+            }
+            $cs = $calendars;
+        }
+
         $changes = array(
             'add' => array(),
             'modify' => array(),
@@ -612,15 +659,16 @@ class Kronolith_Api extends Horde_Registry_Api
      * Return all changes occuring between the specified modification
      * sequences.
      *
-     * @param integer $start  The starting modseq.
-     * @param integer $end    The ending modseq.
+     * @param integer $start             The starting modseq.
+     * @param integer $end               The ending modseq.
+     * @param string|array $calendars    The sources to check. @since 4.2.0
      *
      * @return array  The changes @see getChanges()
      * @since 4.1.1
      */
-    public function getChangesByModSeq($start, $end)
+    public function getChangesByModSeq($start, $end, $calendars = null)
     {
-        return $this->getChanges($start, $end, true, true);
+        return $this->getChanges($start, $end, true, true, $calendars);
     }
 
     /**
@@ -655,12 +703,20 @@ class Kronolith_Api extends Horde_Registry_Api
     /**
      * Return the largest modification sequence from the history backend.
      *
-     * @return integer  The modseq.
+     * @param string $id  The calendar id to return the hightest MDOSEQ for. If
+     *                    null, the highest MODSEQ across all calendars is
+     *                    returned. @since 4.2.0
+     *
+     * @return integer  The MODSEQ value.
      * @since 4.1.1
      */
-    public function getHighestModSeq()
+    public function getHighestModSeq($id = null)
     {
-        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq('kronolith');
+        $parent = 'kronolith';
+        if (!empty($id)) {
+            $parent .= ':' . $id;
+        }
+        return $GLOBALS['injector']->getInstance('Horde_History')->getHighestModSeq($parent);
     }
 
     /**
@@ -723,13 +779,11 @@ class Kronolith_Api extends Horde_Registry_Api
 
             if (count($ids) == 0) {
                 throw new Kronolith_Exception(_("No iCalendar data was found."));
-            } else if (count($ids) == 1) {
-                return $ids[0];
             }
 
             // Now add all the exception instances
             foreach ($recurrences as $recurrence) {
-                $ids[] = $this->_addiCalEvent($recurrence, $kronolith_driver);
+                $ids[] = $this->_addiCalEvent($recurrence, $kronolith_driver, true);
             }
 
             return $ids;
@@ -749,20 +803,24 @@ class Kronolith_Api extends Horde_Registry_Api
      *
      * @param Horde_Icalendar_Vevent $content  The vEvent part
      * @param Kronolith_Driver $driver         The kronolith driver
+     * @param boolean $exception               Content represents an exception
+     *                                         in a recurrence series.
      *
      * @return string  The new event's uid
      */
-    protected function _addiCalEvent($content, $driver)
+    protected function _addiCalEvent($content, $driver, $exception = false)
     {
         $event = $driver->getEvent();
         $event->fromiCalendar($content);
         // Check if the entry already exists in the data source,
         // first by UID.
-        $uid = $event->uid;
-        try {
-            $driver->getByUID($uid, array($driver->calendar));
-            throw new Kronolith_Exception(sprintf(_("%s Already Exists"), $uid));
-        } catch (Horde_Exception $e) {}
+        if (!$exception) {
+            try {
+                $driver->getByUID($uid, array($driver->calendar));
+                throw new Kronolith_Exception(sprintf(_("%s Already Exists"), $event->uid));
+            } catch (Horde_Exception $e) {}
+        }
+
         $result = $driver->search($event);
         // Check if the match really is an exact match:
         if (is_array($result) && count($result) > 0) {
@@ -816,16 +874,18 @@ class Kronolith_Api extends Horde_Registry_Api
      *                                              Still in wide use)
      *                             activesync (Horde_ActiveSync_Message_Appointment)
      *                            </pre>
-     * @param array $optinos      Any additional options to be passed to the
+     * @param array $options      Any additional options to be passed to the
      *                            exporter.
+     * @param array $calendars    Require event to be in these calendars.
+     *                            @since 4.2.0
      *
      * @return string  The requested data.
      * @throws Kronolith_Exception
      * @throws Horde_Exception_NotFound
      */
-    public function export($uid, $contentType, array $options = array())
+    public function export($uid, $contentType, array $options = array(), array $calendars = null)
     {
-        $event = Kronolith::getDriver()->getByUID($uid);
+        $event = Kronolith::getDriver()->getByUID($uid, $calendars);
         if (!$event->hasPermission(Horde_Perms::READ)) {
             throw new Horde_Exception_PermissionDenied();
         }
@@ -917,10 +977,14 @@ class Kronolith_Api extends Horde_Registry_Api
      * @param string $recurrenceId  The reccurenceId for the event instance, if
      *                              this is a deletion of a recurring event
      *                              instance ($uid must not be an array).
+     * @param string $range         The range value if deleting a recurring
+     *                              event instance. Only supported values are
+     *                              null or Kronolith::RANGE_THISANDFUTURE.
+     *                              @since 4.1.5
      *
      * @throws Kronolith_Exception
      */
-    public function delete($uid, $recurrenceId = null)
+    public function delete($uid, $recurrenceId = null, $range = null)
     {
         // Handle an array of UIDs for convenience of deleting multiple events
         // at once.
@@ -933,8 +997,8 @@ class Kronolith_Api extends Horde_Registry_Api
 
         $kronolith_driver = Kronolith::getDriver();
         $events = $kronolith_driver->getByUID($uid, null, true);
-
         $event = null;
+
         if ($GLOBALS['registry']->isAdmin()) {
             $event = $events[0];
         }
@@ -943,7 +1007,7 @@ class Kronolith_Api extends Horde_Registry_Api
         if (empty($event)) {
             $ownerCalendars = Kronolith::listInternalCalendars(true, Horde_Perms::DELETE);
             foreach ($events as $ev) {
-                if ($GLOBALS['registry']->isAdmin() || isset($ownerCalendars[$ev->calendar])) {
+                if (isset($ownerCalendars[$ev->calendar])) {
                     $kronolith_driver->open($ev->calendar);
                     $event = $ev;
                     break;
@@ -967,10 +1031,21 @@ class Kronolith_Api extends Horde_Registry_Api
             throw new Horde_Exception_PermissionDenied();
         }
 
-        if ($recurrenceId && $event->recurs()) {
+        if ($recurrenceId && $event->recurs() && empty($range)) {
             $deleteDate = new Horde_Date($recurrenceId);
             $event->recurrence->addException($deleteDate->format('Y'), $deleteDate->format('m'), $deleteDate->format('d'));
             $event->save();
+        } elseif ($range == Kronolith::RANGE_THISANDFUTURE) {
+            // Deleting the instance and remaining series.
+            $instance = new Horde_Date($recurrenceId);
+            $recurEnd = clone($instance);
+            $recurEnd->mday--;
+            if ($event->end->compareDate($recurEnd) > 0) {
+                 $kronolith_driver->deleteEvent($event->id);
+            } else {
+                 $event->recurrence->setRecurEnd($recurEnd);
+                 $result = $event->save();
+            }
         } elseif ($recurrenceId) {
             throw new Kronolith_Exception(_("Unable to delete event. An exception date was provided but the event does not seem to be recurring."));
         } else {
@@ -990,12 +1065,14 @@ class Kronolith_Api extends Horde_Registry_Api
      *                             text/x-vcalendar
      *                             (Ignored if content is Horde_Icalendar_Vevent)
      *                             activesync (Horde_ActiveSync_Message_Appointment)
+     * @param string $calendar     Ensure the event is replaced in the specified
+     *                             calendar. @since 4.2.0
      *
      * @throws Kronolith_Exception
      */
-    public function replace($uid, $content, $contentType)
+    public function replace($uid, $content, $contentType, $calendar = null)
     {
-        $event = Kronolith::getDriver()->getByUID($uid);
+        $event = Kronolith::getDriver(null, $calendar)->getByUID($uid);
 
         if (!$event->hasPermission(Horde_Perms::EDIT) ||
             ($event->private && $event->creator != $GLOBALS['registry']->getAuth())) {
@@ -1041,11 +1118,16 @@ class Kronolith_Api extends Horde_Registry_Api
             }
         }
 
-        $event->fromiCalendar($component);
-        // Ensure we keep the original UID, even when content does not
-        // contain one and fromiCalendar creates a new one.
-        $event->uid = $uid;
-        $event->save();
+        try {
+            $component->getAttribute('RECURRENCE-ID');
+            $this->_addiCalEvent($component, Kronolith::getDriver(null, $calendar), true);
+        } catch (Horde_Icalendar_Exception $e) {
+            $event->fromiCalendar($component);
+            // Ensure we keep the original UID, even when content does not
+            // contain one and fromiCalendar creates a new one.
+            $event->uid = $uid;
+            $event->save();
+        }
     }
 
     /**
@@ -1166,7 +1248,6 @@ class Kronolith_Api extends Horde_Registry_Api
 
         $found = false;
         $error = _("No attendees have been updated because none of the provided email addresses have been found in the event's attendees list.");
-        $sender_lcase = Horde_String::lower($sender);
 
         foreach ($atnames as $index => $attendee) {
             if ($response->getAttribute('VERSION') < 2) {
@@ -1175,14 +1256,14 @@ class Kronolith_Api extends Horde_Registry_Api
                     continue;
                 }
 
-                $attendee = Horde_String::lower($addr_ob->bare_address);
+                $attendee = $addr_ob->bare_address;
                 $name = $addr_ob->personal;
             } else {
-                $attendee = str_replace('mailto:', '', Horde_String::lower($attendee));
+                $attendee = str_ireplace('mailto:', '', $attendee);
                 $name = isset($atparms[$index]['CN']) ? $atparms[$index]['CN'] : null;
             }
             if ($event->hasAttendee($attendee)) {
-                if (is_null($sender) || $sender_lcase == $attendee) {
+                if (is_null($sender) || $sender == $attendee) {
                     $event->addAttendee($attendee, Kronolith::PART_IGNORE, Kronolith::responseFromICal($atparms[$index]['PARTSTAT']), $name);
                     $found = true;
                 } else {
@@ -1462,4 +1543,94 @@ class Kronolith_Api extends Horde_Registry_Api
         return $return;
     }
 
+    /**
+     * Create a new calendar for the existing user.
+     *
+     * @param string $name  The calendar's display name.
+     * @param array $param  Any additional parameters. May include:
+     *   - color: (string) The color to associate with the calendar.
+     *            DEFAULT: none (color will be randomly assigned).
+     *   - description:  (string) The calendar description.
+     *                   DEFAULT: none (empty description).
+     *   - tags:         (array) An array of tags to apply to the new calendar.
+     *
+     *   - synchronize:   (boolean) If true, add calendar to the list of
+     *                             calendars to syncronize.
+     *                    DEFAULT: false (do not add to the list of calendars).
+     * @return string  The new calendar's UID.
+     * @since 4.2.0
+     */
+    public function addCalendar($name, array $params = array())
+    {
+        global $prefs;
+
+        $info = array(
+            'name' => $name,
+            'color' => empty($params['color']) ? null : $params['color'],
+            'description' => empty($params['description']) ? null : $params['description'],
+            'tags' => empty($params['tags']) ? null : $params['tags']
+        );
+
+        $share = Kronolith::addShare($info);
+
+        if (!empty($params['synchronize'])) {
+            $sync = @unserialize($prefs->getValue('sync_calendars'));
+            $sync[] = $share->getName();
+            $prefs->setValue('sync_calendars', serialize($sync));
+        }
+
+        return $share->getName();
+    }
+
+    /**
+     * Delete the specified calendar.
+     *
+     * @param string $id  The calendar id.
+     */
+    public function deleteCalendar($id)
+    {
+        $calendar = $GLOBALS['injector']
+            ->getInstance('Kronolith_Shares')
+            ->getShare($calendar);
+        Kronolith::deleteShare($calendar);
+    }
+
+    /**
+     * Return an internal calendar.
+     *
+     * @todo Note: This returns a Kronolith_Calendar_Object object instead of a hash
+     * to be consistent with other application APIs. For H6 we need to normalize
+     * the APIs to always return non-objects and/or implement some mechanism to
+     * mark API methods as non-RPC safe.
+     *
+     * @param string $id  The calendar uid (share name).
+     *
+     * @return Kronolith_Calendar The calendar object.
+     * @since 4.2.0
+     */
+    public function getCalendar($id = null)
+    {
+       $driver = Kronolith::getDriver(null, $id);
+       return Kronolith::getCalendar($driver);
+    }
+
+    /**
+     * Update an internal calendar's information.
+     *
+     * @param string $id      The calendar id.
+     * @param array $info     An array of calendar information.
+     *                        @see self::addCalendar()
+     * @since 4.2.0
+     */
+    public function updateCalendar($id, array $info)
+    {
+        $calendar = $this->getCalendar(null, $id);
+
+        // Prevent wiping tags if they were not passed.
+        if (!array_key_exists('tags', $info)) {
+            $info['tags'] = Kronolith::getTagger()->getTags($id, Kronolith_Tagger::TYPE_CALENDAR);
+        }
+        Kronolith::updateShare($calendar->share(), $info);
+    }
+
 }
