@@ -9,7 +9,7 @@
  * - user:     The user name for HTTP Basic Authentication.
  * - password: The password for HTTP Basic Authentication.
  *
- * Copyright 2004-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2004-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -69,9 +69,9 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
      */
     public function backgroundColor()
     {
-        return empty($GLOBALS['all_remote_calendars'][$this->calendar])
-            ? '#dddddd'
-            : $GLOBALS['all_remote_calendars'][$this->calendar]->background();
+        return $GLOBALS['calendar_manager']->getEntry(Kronolith::ALL_REMOTE_CALENDARS, $this->calendar)
+            ? $GLOBALS['calendar_manager']->getEntry(Kronolith::ALL_REMOTE_CALENDARS, $this->calendar)->background()
+            : '#dddddd';
     }
 
     public function listAlarms($date, $fullevent = false)
@@ -114,12 +114,12 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
             return $this->_listCalDAVEvents(
                 $startDate, $endDate, $options['show_recurrence'],
                 $options['has_alarm'], $options['json'],
-                $options['cover_dates']);
+                $options['cover_dates'], $options['hide_exceptions']);
         }
         return $this->_listWebDAVEvents(
             $startDate, $endDate, $options['show_recurrence'],
             $options['has_alarm'], $options['json'],
-            $options['cover_dates']);
+            $options['cover_dates'], $options['hide_exceptions']);
     }
 
     /**
@@ -137,14 +137,17 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
      *                                   toJson() method?
      * @param boolean $coverDates        Whether to add the events to all days
      *                                   that they cover.
+     * $param boolean $hideExceptions    Hide events that represent exceptions
+     *                                   to a recurring event.
      *
      * @return array  Events in the given time range.
      * @throws Kronolith_Exception
      */
-    protected function _listWebDAVEvents($startDate = null, $endDate = null,
-                                         $showRecurrence = false,
-                                         $hasAlarm = false, $json = false,
-                                         $coverDates = true)
+    protected function _listWebDAVEvents(
+        $startDate = null, $endDate = null, $showRecurrence = false,
+        $hasAlarm = false, $json = false, $coverDates = true,
+        $hideExceptions = false
+    )
     {
         $ical = $this->getRemoteCalendar();
 
@@ -166,8 +169,10 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
         $endDate->min = $endDate->sec = 59;
 
         $results = array();
-        $this->_processComponents($results, $ical, $startDate, $endDate,
-                                  $showRecurrence, $json, $coverDates);
+        $this->_processComponents(
+            $results, $ical, $startDate, $endDate, $showRecurrence, $json,
+            $coverDates, $hideExceptions
+        );
 
         return $results;
     }
@@ -187,14 +192,17 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
      *                                   toJson() method?
      * @param boolean $coverDates        Whether to add the events to all days
      *                                   that they cover.
+     * $param boolean $hideExceptions    Hide events that represent exceptions
+     *                                   to a recurring event.
      *
      * @return array  Events in the given time range.
      * @throws Kronolith_Exception
      */
-    protected function _listCalDAVEvents($startDate = null, $endDate = null,
-                                         $showRecurrence = false,
-                                         $hasAlarm = false, $json = false,
-                                         $coverDates = true)
+    protected function _listCalDAVEvents(
+        $startDate = null, $endDate = null, $showRecurrence = false,
+        $hasAlarm = false, $json = false, $coverDates = true,
+        $hideExceptions = false
+    )
     {
         if (!is_null($startDate)) {
             $startDate = clone $startDate;
@@ -265,9 +273,11 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
             } catch (Horde_Icalendar_Exception $e) {
                 throw new Kronolith_Exception($e);
             }
-            $this->_processComponents($results, $ical, $startDate, $endDate,
-                                      $showRecurrence, $json, $coverDates,
-                                      trim(str_replace($path, '', $response->href), '/'));
+            $this->_processComponents(
+                $results, $ical, $startDate, $endDate, $showRecurrence, $json,
+                $coverDates, $hideExceptions,
+                trim(str_replace($path, '', $response->href), '/')
+            );
         }
 
         return $results;
@@ -290,13 +300,16 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
      *                                   toJson() method?
      * @param boolean $coverDates        Whether to add the events to all days
      *                                   that they cover.
+     * $param boolean $hideExceptions    Hide events that represent exceptions
+     *                                   to a recurring event.
      * @param string $id                 Enforce a certain event id (not UID).
      *
      * @throws Kronolith_Exception
      */
-    protected function _processComponents(&$results, $ical, $startDate,
-                                          $endDate, $showRecurrence, $json,
-                                          $coverDates, $id = null)
+    protected function _processComponents(
+        &$results, $ical, $startDate, $endDate, $showRecurrence, $json,
+        $coverDates, $hideExceptions, $id = null
+    )
     {
         $components = $ical->getComponents();
         $events = array();
@@ -305,10 +318,8 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
         for ($i = 0; $i < $count; $i++) {
             $component = $components[$i];
             if ($component->getType() == 'vEvent') {
-                $event = new Kronolith_Event_Ical($this);
-                $event->status = Kronolith::STATUS_FREE;
+                $event = new Kronolith_Event_Ical($this, $component);
                 $event->permission = $this->getPermission();
-                $event->fromDriver($component);
                 // Force string so JSON encoding is consistent across drivers.
                 $event->id = $id ? $id : 'ical' . $i;
 
@@ -320,6 +331,9 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
                         is_string($uid = $component->getAttribute('UID')) &&
                         is_int($seq = $component->getAttribute('SEQUENCE'))) {
                         $exceptions[$uid][$seq] = $recurrence_id;
+                        if ($hideExceptions) {
+                            continue;
+                        }
                         $event->id .= '/' . $recurrence_id;
                     }
                 } catch (Horde_Icalendar_Exception $e) {}
@@ -340,7 +354,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
                     if ($event->recurrence->hasRecurEnd() &&
                         $event->recurrence->recurEnd->compareDateTime($startDate) < 0) {
                         continue;
-                    } else if ($endDate) {
+                    } elseif ($endDate) {
                         $next = $event->recurrence->nextRecurrence($startDate);
                         if ($next == false || $next->compareDateTime($endDate) > 0) {
                             continue;
@@ -411,10 +425,8 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
         $components = $ical->getComponents();
         if (isset($components[$eventId]) &&
             $components[$eventId]->getType() == 'vEvent') {
-            $event = new Kronolith_Event_Ical($this);
-            $event->status = Kronolith::STATUS_FREE;
+            $event = new Kronolith_Event_Ical($this, $components[$eventId]);
             $event->permission = $this->getPermission();
-            $event->fromDriver($components[$eventId]);
             $event->id = 'ical' . $eventId;
             return $event;
         }
@@ -435,7 +447,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
     {
         $response = $this->_saveEvent($event);
         if (!in_array($response['statusCode'], array(200, 204))) {
-            Horde::logMessage(sprintf('Failed to update event on remote calendar: url = "%s", status = %s',
+            Horde::log(sprintf('Failed to update event on remote calendar: url = "%s", status = %s',
                                       $response['url'], $response['statusCode']), 'INFO');
             throw new Kronolith_Exception(_("The event could not be updated on the remote server."));
         }
@@ -462,7 +474,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
 
         $response = $this->_saveEvent($event);
         if (!in_array($response['statusCode'], array(200, 201, 204))) {
-            Horde::logMessage(sprintf('Failed to create event on remote calendar: status = %s',
+            Horde::log(sprintf('Failed to create event on remote calendar: status = %s',
                                       $response['statusCode']), 'INFO');
             throw new Kronolith_Exception(_("The event could not be added to the remote server."));
         }
@@ -493,7 +505,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
                     array('Content-Type' => 'text/calendar')
                 );
         } catch (Horde_Dav_Exception $e) {
-            Horde::logMessage($e, 'INFO');
+            Horde::log($e, 'INFO');
             throw new Kronolith_Exception($e);
         }
     }
@@ -531,11 +543,11 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
         try {
             $response = $this->_getClient($url)->request('DELETE');
         } catch (Horde_Dav_Exception $e) {
-            Horde::logMessage($e, 'INFO');
+            Horde::log($e, 'INFO');
             throw new Kronolith_Exception($e);
         }
         if (!in_array($response['statusCode'], array(200, 202, 204))) {
-            Horde::logMessage(sprintf('Failed to delete event from remote calendar: url = "%s", status = %s',
+            Horde::log(sprintf('Failed to delete event from remote calendar: url = "%s", status = %s',
                                       $url, $response['statusCode']), 'INFO');
             throw new Kronolith_Exception(_("The event could not be deleted from the remote server."));
         }
@@ -574,7 +586,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
                 throw new Horde_Dav_Exception('Request Failed', $response['statusCode']);
             }
         } catch (Horde_Dav_Exception $e) {
-            Horde::logMessage(
+            Horde::log(
                 sprintf('Failed to retrieve remote calendar: url = "%s", status = %s',
                         $url, $e->getCode()),
                 'INFO'
@@ -590,7 +602,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
         }
 
         /* Log fetch at DEBUG level. */
-        Horde::logMessage(
+        Horde::log(
             sprintf('Retrieved remote calendar for %s: url = "%s"',
                     $GLOBALS['registry']->getAuth(), $url),
             'DEBUG'
@@ -633,7 +645,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
             $this->_davSupport = $client->options();
         } catch (Horde_Dav_Exception $e) {
             if ($e->getCode() != 405) {
-                Horde::logMessage($e, 'INFO');
+                Horde::log($e, 'INFO');
             }
             return false;
         }
@@ -652,9 +664,12 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
             $properties = $client->propfind(
                 '',
                 array('{DAV:}resourcetype', '{DAV:}current-user-privilege-set')
-            );
+             );
         } catch (\Sabre\DAV\Exception $e) {
-            Horde::logMessage($e, 'INFO');
+            Horde::log($e, 'INFO');
+            return false;
+        } catch (Horde_Dav_Exception $e) {
+            Horde::log($e, 'INFO');
             return false;
         }
 
@@ -722,7 +737,7 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
                                             'Content-Type' => 'application/xml'),
                                       $headers));
         } catch (Horde_Dav_Exception $e) {
-            Horde::logMessage($e, 'INFO');
+            Horde::log($e, 'INFO');
             throw new Kronolith_Exception($e);
         }
         if ($response['statusCode'] != 207) {
@@ -765,20 +780,21 @@ class Kronolith_Driver_Ical extends Kronolith_Driver
      */
     protected function _getClient($uri)
     {
-        $options = array('request.timeout' => isset($this->_params['timeout'])
-                                              ? $this->_params['timeout']
-                                              : 5);
+        $hordeOptions = array(
+            'request.timeout' => isset($this->_params['timeout'])
+                ? $this->_params['timeout']
+                : 5
+        );
+        $sabreOptions = array('baseUri' => $uri);
         if (!empty($this->_params['user'])) {
-            $options['request.username'] = $this->_params['user'];
-            $options['request.password'] = $this->_params['password'];
+            $hordeOptions['request.username'] = $sabreOptions['userName'] = $this->_params['user'];
+            $hordeOptions['request.password'] = $sabreOptions['password'] = $this->_params['password'];
         }
+        $sabreOptions['client'] = $GLOBALS['injector']
+            ->getInstance('Horde_Core_Factory_HttpClient')
+            ->create($hordeOptions);
 
-        $this->_client = new Horde_Dav_Client(array(
-            'client' => $GLOBALS['injector']
-                ->getInstance('Horde_Core_Factory_HttpClient')
-                ->create($options),
-            'baseUri' => $uri
-        ));
+        $this->_client = new Horde_Dav_Client($sabreOptions);
 
         return $this->_client;
     }
