@@ -3,7 +3,7 @@
  * The Kronolith_Driver_Sql class implements the Kronolith_Driver API for a
  * SQL backend.
  *
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -22,6 +22,13 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
      */
     protected $_db;
 
+    /**
+     * Column information as Horde_Db_Adapter_Base_Column objects.
+     *
+     * @var array
+     */
+    protected $_columns = array();
+
     /**
      * Cache events as we fetch them to avoid fetching the same event from the
      * DB twice.
@@ -46,8 +53,8 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
      */
     public function backgroundColor()
     {
-        if (isset($GLOBALS['all_calendars'][$this->calendar])) {
-            return $GLOBALS['all_calendars'][$this->calendar]->background();
+        if ($GLOBALS['calendar_manager']->getEntry(Kronolith::ALL_CALENDARS, $this->calendar) !== false) {
+            return $GLOBALS['calendar_manager']->getEntry(Kronolith::ALL_CALENDARS, $this->calendar)->background();
         }
         return '#dddddd';
     }
@@ -374,8 +381,8 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
 
         $events = array();
         foreach ($qr as $row) {
-            /* If the event did not have a UID before, we need to give
-             * it one. */
+            /* If the event did not have a UID before, we need to give it
+             * one. */
             if (empty($row['event_uid'])) {
                 $row['event_uid'] = (string)new Horde_Support_Guid;
 
@@ -389,6 +396,9 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
                 }
             }
 
+            /* Convert TEXT/CLOB fields. */
+            $row = $this->convertBlobs($row);
+
             /* We have all the information we need to create an event object
              * for this event, so go ahead and cache it. */
             $this->_cache[$this->calendar][$row['event_id']] = new $this->_eventClass($this, $row);
@@ -460,6 +470,9 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
             throw new Kronolith_Exception($e);
         }
         if ($event) {
+            /* Convert TEXT/CLOB fields. */
+            $event = $this->convertBlobs($event);
+
             $this->_cache[$this->calendar][$eventId] = new $this->_eventClass($this, $event);
             return $this->_cache[$this->calendar][$eventId];
         }
@@ -492,7 +505,7 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         $values = array((string)$uid);
 
         /* Optionally filter by calendar */
-        if (!is_null($calendars)) {
+        if (!empty($calendars)) {
             if (!count($calendars)) {
                 throw new Kronolith_Exception(_("No calendars to search"));
             }
@@ -506,11 +519,14 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
             throw new Kronolith_Exception($e);
         }
         if (!count($events)) {
-            throw new Horde_Exception_NotFound($uid . ' not found');
+            throw new Horde_Exception_NotFound(sprintf(_("%s not found"), $uid));
         }
 
         $eventArray = array();
         foreach ($events as $event) {
+            /* Convert TEXT/CLOB fields. */
+            $event = $this->convertBlobs($event);
+
             $this->open($event['calendar_id']);
             $this->_cache[$this->calendar][$event['event_id']] = new $this->_eventClass($this, $event);
             $eventArray[] = $this->_cache[$this->calendar][$event['event_id']];
@@ -574,9 +590,10 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         }
 
         foreach (array_keys($newProperties) as $property) {
-            if (empty($oldProperties[$property]) || ($oldProperties[$property] != $newProperties[$property])) {
+            if (!isset($oldProperties[$property]) ||
+                ($oldProperties[$property] != $newProperties[$property])) {
                 $changes['new'][$property] = $newProperties[$property];
-                $changes['old'][$property] = !empty($oldProperties[$property]) ? $oldProperties[$property] : null;
+                $changes['old'][$property] = !isset($oldProperties[$property]) ? $oldProperties[$property] : null;
             }
         }
 
@@ -615,9 +632,9 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         /* Log the modification of this item in the history log. */
         if ($event->uid) {
             try {
-                $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->uid, $history, false);
+                $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->uid, $history, true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -626,9 +643,9 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
          * change. */
         if ($event->baseid) {
             try {
-                $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->baseid, $history, false);
+                $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->baseid, $history, true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
         $this->_updateTags($event);
@@ -687,7 +704,7 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         try {
             $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->uid, array('action' => 'add'), true);
         } catch (Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
 
         $this->_addTags($event);
@@ -746,7 +763,7 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
     {
         $oldCalendar = $this->calendar;
         $this->open($calendar);
-        $events = $this->listEvents(null, null, array('cover_dates' => false));
+        $events = $this->listEvents(null, null, array('cover_dates' => false, 'hide_exceptions' => true));
         $uids = array();
         foreach ($events as $dayevents) {
             foreach ($dayevents as $event) {
@@ -755,11 +772,7 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         }
         foreach ($uids as $uid) {
             $event = $this->getByUID($uid, array($calendar));
-            try {
-                $this->deleteEvent($event->id);
-            } catch (Kronolith_Exception $e) {
-                Horde::logMessage($e, 'ERR');
-            }
+            $this->deleteEvent($event->id);
         }
 
         $this->open($oldCalendar);
@@ -861,6 +874,8 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         $this->_params = array_merge(array(
             'table' => 'kronolith_events'
         ), $this->_params);
+
+        $this->_columns = $this->_db->columns($this->_params['table']);
     }
 
     /**
@@ -889,4 +904,23 @@ class Kronolith_Driver_Sql extends Kronolith_Driver
         return Horde_String::convertCharset($value, 'UTF-8', $this->_params['charset']);
     }
 
+    /**
+     * Converts TEXT/CLOB fields in an event.
+     *
+     * @param array $event  An event hash with TEXT/CLOB columns.
+     *
+     * @return array  The event with TEXT/CLOB columns converted to strings.
+     */
+    public function convertBlobs($event)
+    {
+        $clobs = array(
+            'alarm_methods', 'attendees', 'description', 'exceptions',
+            'location', 'resources'
+        );
+        foreach ($clobs as $clob) {
+            $event['event_' . $clob] = $this->_columns['event_' . $clob]
+                ->binaryToString($event['event_' . $clob]);
+        }
+        return $event;
+    }
 }
