@@ -15,7 +15,7 @@
 /**
  * Test the Kolab driver.
  *
- * Copyright 2011-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPLv2). If you did not
  * receive this file, see http://www.horde.org/licenses/gpl
@@ -43,6 +43,6 @@ class Kronolith_Integration_Driver_KolabTest extends Kronolith_Integration_Drive
 
     public function setUp()
     {
-        $this->markTestSkipped('Unserialization error from Kolab share objects.');
+        $this->markTestIncomplete('Unserialization error from Kolab share objects.');
     }
 }
