@@ -15,7 +15,7 @@
 /**
  * Test the core Kronolith class with the Kolab backend.
  *
- * Copyright 2011-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPLv2). If you did not
  * receive this file, see http://www.horde.org/licenses/gpl
@@ -48,6 +48,6 @@ class Kronolith_Integration_Kronolith_KolabTest extends Kronolith_Integration_Kr
 
     public function setUp()
     {
-        $this->markTestSkipped("No query of type 'Share' registered!");
+        $this->markTestIncomplete("No query of type 'Share' registered!");
     }
 }
